<?php
// +----------------------------------------------------------------------
// | SentCMS [ WE CAN DO IT JUST THINK IT ]
// +----------------------------------------------------------------------
// | Copyright (c) 2013 http://www.tensent.cn All rights reserved.
// +----------------------------------------------------------------------
// | Author: molong <molong@tensent.cn> <http://www.tensent.cn>
// +----------------------------------------------------------------------

namespace app\admin\controller;

/**
 * @title 配置管理
 */
class Config extends Base
{

    private $model;

    protected function initialize()
    {
        parent::initialize();
        $this->model = model('Config');
    }

    /**
     * @title 配置管理
     * @author 麦当苗儿 <zuojiazi@vip.qq.com>
     */
    public function index()
    {
        $group = input('group', 0, 'trim');
        $name = input('name', '', 'trim');
        /* 查询条件初始化 */
        $map = array();
        if ($group) {
            $map['group'] = $group;
        }

        if ($name) {
            $map['name'] = array('like', '%' . $name . '%');
        }

        $order = ['group1' => 'desc', 'group2'];

        $list = $this->model->where($map)->order($order)->select();
        foreach ($list as $item){
            $group1 = ['system' => '系统', 'social' => '社保比例', 'basic' => '缴费基数', 'service' => '服务费', 'fund' => '公积金比例', 'others' => '手续费'];
            $group2 = ['system' => '系统', 'farmer' => '农户', 'town' => '城镇户'];
            $item['group1'] = $group1[$item['group1']];
            $item['group2'] = $group2[$item['group2']];
        }
        // 记录当前列表页的cookie
        Cookie('__forward__', $_SERVER['REQUEST_URI']);

        $data = array(
            'list' => $list,
        );

        $this->assign($data);
        $this->setMeta('配置管理');
        return $this->fetch();
    }

    /**
     * @title 编辑字段
     * @param null $name
     * @param null $value
     * @param null $pk
     * @return bool|int
     */
    public function editable($name = null, $value = null, $pk = null)
    {
        if ($name && ($value != null || $value != '') && $pk) {
            return db('Config')->where(array('id' => $pk))->setField($name, $value);
        }
        return false;
    }


    /**
     * @title 信息配置
     */
    public function group($id = 1)
    {
        if ($this->request->isPost()) {
            $config = $this->request->post('config/a');
            $model = model('Config');
            foreach ($config as $key => $value) {
                $model->where(array('name' => $key))->setField('value', $value);
            }
            //清除db_config_data缓存
            cache('db_config_data', null);
            return $this->success("更新成功！");
        } else {
            $type = config('config_group_list');
            $list = db("Config")->where(array('status' => 1, 'group' => $id))->field('id,name,title,extra,value,remark,type')->order('sort')->select();
            if ($list) {
                $this->assign('list', $list);
            }
            $this->assign('id', $id);
            $this->setMeta($type[$id] . '设置');
            return $this->fetch();
        }
    }

    /**
     * @title 新增配置
     * @author 麦当苗儿 <zuojiazi@vip.qq.com>
     */
    public function add()
    {
        if ($this->request->isPost()) {
            $config = model('Config');
            $data = $this->request->post();
            if ($data) {
                $id = $config->validate(true)->save($data);
                if ($id) {
                    cache('db_config_data', null);
                    //记录行为
                    action_log('update_config', 'config', $id, session('user_auth.uid'));
                    return $this->success('新增成功', url('index'));
                } else {
                    return $this->error('新增失败');
                }
            } else {
                return $this->error($config->getError());
            }
        } else {
            $this->setMeta('新增配置');
            $this->assign('info', null);
            return $this->fetch('edit');
        }
    }

    /**
     * @title 编辑配置
     * @author 麦当苗儿 <zuojiazi@vip.qq.com>
     */
    public function edit($id = 0)
    {
        if ($this->request->isPost()) {
            $config = model('Config');
            $data = $this->request->post();
            if ($data) {
                $result = $config->validate('Config.edit')->save($data, array('id' => $data['id']));
                if (false !== $result) {
                    cache('db_config_data', null);
                    //记录行为
                    action_log('update_config', 'config', $data['id'], session('user_auth.uid'));
                    return $this->success('更新成功', Cookie('__forward__'));
                } else {
                    return $this->error($config->getError(), '');
                }
            } else {
                return $this->error($config->getError());
            }
        } else {
            $info = array();
            /* 获取数据 */
            $info = db('Config')->field(true)->find($id);

            if (false === $info) {
                return $this->error('获取配置信息错误');
            }
            $this->assign('info', $info);
            $this->setMeta('编辑配置');
            return $this->fetch();
        }
    }

    /**
     * @title 批量保存配置
     * @author 麦当苗儿 <zuojiazi@vip.qq.com>
     */
    public function save($config)
    {
        if ($config && is_array($config)) {
            $Config = db('Config');
            foreach ($config as $name => $value) {
                $map = array('name' => $name);
                $Config->where($map)->setField('value', $value);
            }
        }
        cache('db_config_data', null);
        return $this->success('保存成功！');
    }

    /**
     * @title 删除配置
     * @author 麦当苗儿 <zuojiazi@vip.qq.com>
     */
    public function del()
    {
        $id = array_unique((array)input('id', 0));

        if (empty($id)) {
            return $this->error('请选择要操作的数据!');
        }

        $map = array('id' => array('in', $id));
        if (db('Config')->where($map)->delete()) {
            cache('DB_CONFIG_DATA', null);
            //记录行为
            action_log('update_config', 'config', $id, session('user_auth.uid'));
            return $this->success('删除成功');
        } else {
            return $this->error('删除失败！');
        }
    }

    /**
     * @title 配置排序
     * @author huajie <banhuajie@163.com>
     */
    public function sort()
    {
        if ($this->request->isGet()) {
            $ids = input('ids');
            //获取排序的数据
            $map = array('status' => array('gt', -1));
            if (!empty($ids)) {
                $map['id'] = array('in', $ids);
            } elseif (input('group')) {
                $map['group'] = input('group');
            }
            $list = db('Config')->where($map)->field('id,title')->order('sort asc,id asc')->select();

            $this->assign('list', $list);
            $this->setMeta('配置排序');
            return $this->fetch();
        } elseif ($this->request->isPost()) {
            $ids = input('post.ids');
            $ids = explode(',', $ids);
            foreach ($ids as $key => $value) {
                $res = db('Config')->where(array('id' => $value))->setField('sort', $key + 1);
            }
            if ($res !== false) {
                return $this->success('排序成功！', Cookie('__forward__'));
            } else {
                return $this->error('排序失败！');
            }
        } else {
            return $this->error('非法请求！');
        }
    }

    /**
     * @title 主题选择
     */
    public function themes()
    {
        $list = $this->model->getThemesList();
        $pc = config('pc_themes');
        $mobile = config('mobile_themes');
        $data = array(
            'pc' => $pc,
            'mobile' => $mobile,
            'list' => $list,
        );

        $this->assign($data);
        $this->setMeta('主题设置');
        return $this->fetch();
    }

    /**
     * @title 设置主题
     * @return json
     */
    public function setthemes($name, $id)
    {
        $result = db('Config')->where('name', $name . '_themes')->setField('value', $id);
        if (false !== $result) {
            \think\Cache::clear();
            return $this->success('设置成功！');
        } else {
            return $this->error('设置失败！');
        }
    }
}