<?php
/**
 * Created by PhpStorm.
 * MiniUser: Administrator
 * Date: 2018/12/8
 * Time: 21:35
 */

namespace app\admin\controller;

use app\common\model\ChannelActive;
use think\Db;
use think\facade\Log;

/**
 * Class Mini
 * @title 小程序
 * @package app\admin\controller
 */
class Mini extends Base
{
    protected $model;
    protected $miniGroup;

    protected $beforeActionList = ['beforeMethod'];

    protected function beforeMethod()
    {
        $this->model = model('Mini');
        $this->miniGroup = getUserType();
    }

    /**
     * @title 小程序列表
     * @return mixed
     */
    public function index()
    {
        $list = $this->model->order('mid')->where('status', 1)
            ->paginate(config('siteinfo.list_rows'), false);
        $data = array(
            'group' => $this->miniGroup,
            'list' => $list,
            'page' => $list->render(),
            'keyList' => $this->model->keyList
        );
        $this->setMeta('小程序列表');
        $this->assign($data);
        return $this->fetch('mini/index');
    }

    /**
     * @title 获取应用编码
     * @param $id
     */
    public function getmidcode($id)
    {
        return json(encodeN($id, config('siteinfo.m_param_salt')));
    }

    /**
     * @title 清除应用配置
     * @param $id
     */
    public function rmconf($id)
    {
        $redis = new \think\cache\driver\Redis(array(
            'select' => explode('.',$this->request->host())[0] == 'test'? 63:$id,
            'timeout' => 300
        ));
        $result1 = $redis->dels('config_file_*');
        $redis->select(63);
        $result2 = $redis->dels('config_file_*');
        if ($result1 && $result2) {
            $this->success('请求成功');
        } else {
            $this->error('请求失败');
        }
    }

    /**
     * @title 渠道推广
     * @return mixed
     * @throws \think\exception\DbException
     */
    public function channel()
    {
        $list = Db::view('ChannelActive', ['aid' => 'id', 'name', 'path', 'create_timestamp'])
            ->view('SysAdmin', ['nickname' => 'sname'], 'ChannelActive.sid=SysAdmin.sid')
            ->view('Mini', ['name' => 'mname'], 'ChannelActive.mid=Mini.mid')
            ->paginate(config('siteinfo.list_rows'), false);
        $data = array(
            'list' => $list,
            'page' => $list->render(),
        );
        $this->setMeta('推广活动');
        $this->assign($data);
        return $this->fetch();
    }

    /**
     * @title 新增应用
     * @return bool|mixed
     */
    public function addMini()
    {
        if ($this->request->isPost()) {
            $data = $this->request->param();
            $data['sid'] = $this->miniGroup;
            $valid = $this->validate($data, 'Mini');
            if ($valid !== true) {
                $this->error('验证失败：' . $valid);
            }
            $result = $this->model->save($data);
            if ($result) {
                $this->success('新增成功', 'admin/mini/index');
            } else {
                $this->error('新增失败');
            }
        } else {
            $data = array(
                'keyList' => $this->model->keyList,
            );
            $this->assign($data);
            $this->setMeta('新增小程序');
            return $this->fetch('public/edit');
        }
        return false;
    }

    /**
     * @title 修改应用
     * @param int $id
     * @return bool|mixed
     */
    public function editMini($id = 0)
    {
        if ($this->request->isPost()) {
            $data = $this->request->param();
            try {
                $this->model->save($data, array('mid' => $data['id']));
                $this->success('更新成功', 'admin/mini/index');
            } catch (\think\Exception\DbException $e) {
                $this->error('更新失败：' . $e->getMessage());
            }
        } else {
            try {
                $info = $this->model->get($id);
            } catch (\think\Exception $e) {
                $this->error('获取后台小程序信息错误：' . $e->getMessage());
            }
            $data = array(
                'info' => $info,
                'keyList' => $this->model->keyList,
            );
            $this->assign($data);
            $this->setMeta('编辑后台小程序');
            return $this->fetch('public/edit');
        }
        return false;
    }

    /**
     * @title 编辑字段
     * @param null $name
     * @param null $value
     * @param null $pk
     * @return bool|int
     */
    public function editable($name = null, $value = null, $pk = null)
    {
        if ($name && ($value != null || $value != '') && $pk) {
            return db('Mini')->where(array('mid' => $pk))->setField($name, $value);
        }
        return false;
    }

    /**
     * @title 删除小程序
     * @param $id
     */
    public function delMini($id)
    {
        if (empty($id)) {
            $this->error('请选择要操作的数据!');
        }
        if ($this->model->save(['status' => 0], ['mid' => $id])) {
            $this->success('删除成功');
        } else {
            $this->error('删除失败！');
        }
    }

    /**
     * @title 添加推广活动
     * @return bool|mixed
     */
    public function addActive()
    {
        if ($this->request->isPost()) {
            $channelActive = model('ChannelActive');
            $data = $this->request->param();
            Db::startTrans();
            try {
                $channelActive->save($data);
                $aid = $channelActive->getLastInsID();
                $params = getSurveyParam($data['mid'], $aid);
                $path = $data['path'] ?: 'pages/index/index';
                $path = stripos($path, '?') !== false ? $path . '&' . $params : $path . '?' . $params;
                $result = model('ChannelActive')->save(['path' => $path], ['aid' => $aid]);
                Db::commit();
            } catch (\Exception $e) {
                // 回滚事务
                Db::rollback();
                return $e;
            }
            if ($result) {
                $this->success('新增成功', 'admin/mini/channel');
            } else {
                $this->error('新增失败');
            }
        } else {
            $data = array(
                'keyList' => ChannelActive::getKeyList(),
            );
            $this->assign($data);
            $this->setMeta('新增推广活动');
            return $this->fetch('public/edit');
        }
        return false;
    }

    /**
     * @title 删除推广活动
     * @param $id
     * @throws \Exception
     */
    public function delActive($id)
    {
        if (empty($id)) {
            $this->error('请选择要操作的数据!');
        }
        if (model('ChannelActive')->save(['status' => 0], ['aid' => $id])) {
            $this->success('删除成功');
        } else {
            $this->error('删除失败！');
        }
    }

    /**
     * @title 获取微信二维码
     * @param $id
     * @return $this
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\ModelNotFoundException
     * @throws \think\exception\DbException
     */
    public function getWXAQRCode($id)
    {
        $dir = config('siteinfo.uploads_dir');
        mkdirs(config('siteinfo.uploads_dir'));
        $filename = md5('wxacode' . $id) . '.jpg';
        if (!@getimagesize($dir . $filename)) {
            $this->createWXAQRCode($id, $dir . $filename);
        }
        return download('./uploads/wxacode/' . $filename, 'wxcode')->mimeType('image/jpeg');
    }


    /**
     * @title 生成小程序码
     * @param $id
     * @param $filename
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\ModelNotFoundException
     * @throws \think\exception\DbException
     */
    private function createWXAQRCode($id, $filename)
    {
        $channelActive = model('ChannelActive')->where('aid', $id)->field('mid,path')->find();
        $mini = model('mini')->where('mid', '=', $channelActive['mid'])->field('app_id,app_secret')->find();
        $data = array(
            'path' => $channelActive['path'],
            'width' => 512,
        );
        $wx = new \wx\MiniApi($mini['app_id'], $mini['app_secret'], $mini['offer_id'], $mini['midas_secret']);
        $data = $wx->createWXAQRCode($data, 'a');
        saveImgage($filename, $data);
    }

// +----------------------------------------------------------------------
// | 更新与新增
// +----------------------------------------------------------------------
// | 0、小程序用户列表；
// | 1、小程序订单列表；
// | 2、小程序红包列表；
// | 3、小程序提现操作curl；
// +----------------------------------------------------------------------
// | 2019-01-18
// +----------------------------------------------------------------------

    /**
     * 订单列表
     * @return mixed
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\ModelNotFoundException
     * @throws \think\exception\DbException
     */
    public function users()
    {
        $data = $this->request->param();
        $map = [];

        if (isset($data['type']) && $data['type'] != '') {
            $map[] = ['orders.type', '=', $data['type']];
        }
        if (isset($data['status']) && $data['status'] != '') {
            $map[] = ['orders.status', '=', $data['status']];
        }

        $minilist = Db::view('mini', 'mid,name')
            ->view('orders', ['count(*)' => 'transfer_num'], 'orders.mid=mini.mid', 'left')
            ->cache()->select();

        $mid = $minilist[0]['mid'];

        if (isset($data['id'])) {
            $mid = $data['id'];
        }

        $userinfo = db('user_info_' . $mid)
            ->field('uid,nickname,avator,sex,balance,fragment,data,last_login_timestamp,create_timestamp')
            ->where($map)
            ->paginate(20, true);
        $page = $userinfo->render();

        foreach ($userinfo as $i => $user) {
            $user['sex'] = $user['sex'] == 1 ? '男' : '女';
            $data = array_merge(json_decode('{"userBase":{"coin":237,"money":0,"power":51,"powerMax":60,"star":3,"level":1,"zpfree":0,"zpuseNum":"3","redsuiNum":0,"limitedtime":1548074542968,"limitedgetDate":1548074542968,"isgetlimitedget":false,"zpreward_id":4,"powerDate":1548075119223},"roles":{"1":{"name":1,"chineseName":"\u5c0f\u767d\u9f99","spType":1,"ATTBase":8,"DEFBase":0,"HPBase":10,"SPEEDBase":4.5,"STARBase":1,"SKILL":101,"locked":0,"unLockCostType":0,"unLockCost":0,"CD":2,"heroType":1,"bulletType":1,"bulletSpeed":10,"DEFLevel":0,"HPLevel":0,"ATTLevel":0,"STARLevel":0},"2":{"name":2,"chineseName":"\u516b\u6212","spType":2,"ATTBase":7,"DEFBase":1,"HPBase":11,"SPEEDBase":4,"STARBase":1,"SKILL":201,"locked":0,"unLockCostType":0,"unLockCost":0,"CD":4,"heroType":2,"bulletType":2,"bulletSpeed":10,"DEFLevel":0,"HPLevel":0,"ATTLevel":0,"STARLevel":0},"3":{"name":3,"chineseName":"\u6c99\u50e7","spType":3,"ATTBase":13,"DEFBase":2,"HPBase":14,"SPEEDBase":4.5,"STARBase":1,"SKILL":301,"locked":1,"unLockCostType":0,"unLockCost":3000,"CD":5,"heroType":3,"bulletType":3,"bulletSpeed":10,"DEFLevel":0,"HPLevel":0,"ATTLevel":0,"STARLevel":0},"4":{"name":4,"chineseName":"\u5b59\u609f\u7a7a","spType":4,"ATTBase":14,"DEFBase":4,"HPBase":16,"SPEEDBase":5,"STARBase":1,"SKILL":401,"locked":1,"unLockCostType":3,"unLockCost":0,"CD":6,"heroType":4,"bulletType":4,"bulletSpeed":10,"DEFLevel":0,"HPLevel":0,"ATTLevel":0,"STARLevel":0},"5":{"name":5,"chineseName":"\u767d\u9aa8\u7cbe","spType":5,"ATTBase":15,"DEFBase":3,"HPBase":15,"SPEEDBase":5,"STARBase":1,"SKILL":501,"locked":1,"unLockCostType":1,"unLockCost":2000,"CD":5,"heroType":5,"bulletType":5,"bulletSpeed":10,"DEFLevel":0,"HPLevel":0,"ATTLevel":0,"STARLevel":0}},"levelMap":{"1":3},"loginTaskgetDate":0,"loginTaskgetNum":0,"gold":237,"energy":51}', true), json_decode($user['data'], true));
            trace($data, 'alert');
            $user['data'] = Array(
                'coin' => $data['userBase']['coin'],
                'money' => $data['userBase']['money'],
                'power' => $data['userBase']['power'],
                'star' => $data['userBase']['star'],
                'level' => $data['userBase']['level'],
            );
            foreach ($data['roles'] as $role) {
                $user['data']['roles'][] = array(
                    'chineseName' => $role['chineseName'],
                    'ATTLevel' => $role['ATTLevel'],
                    'DEFLevel' => $role['DEFLevel'],
                    'HPLevel' => $role['HPLevel'],
                    'STARBase' => $role['STARBase']
                );
            }
        }

        $this->assign(array(
            'minilist' => $minilist,
            'userlist' => $userinfo,
            'cmini' => $mid,
            'page' => $page,
        ));
        return $this->fetch('user_list');
    }

    /**
     * 订单列表
     * @return mixed
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\ModelNotFoundException
     * @throws \think\exception\DbException
     */
    public function orders()
    {
        $data = $this->request->param();
        $map = [];
        if (isset($data['type']) && $data['type'] != '') {
            $map[] = ['orders.type', '=', $data['type']];
        }
        if (isset($data['status']) && $data['status'] != '') {
            $map[] = ['orders.status', '=', $data['status']];
        }

        $minilist = Db::view('mini', 'mid,name')
            ->view('orders', ['count(*)' => 'transfer_num'], 'orders.mid=mini.mid', 'left')
            ->cache()->select();

        $mid = $minilist[0]['mid'];

        if (isset($data['id'])) {
            $mid = $data['id'];
        }

        $orderinfo = Db::view('orders', 'order_id,order_sn,type,price,amount,status,create_timestamp')
            ->view('user_info_' . $mid, 'nickname,avator', 'orders.uid=user_info_' . $mid . '.uid', 'left')
            ->where($map)
            ->select();

        $this->assign(array(
            'minilist' => $minilist,
            'orderlist' => $orderinfo,
            'cmini' => $mid,
        ));
        return $this->fetch('order_list');
    }

    /**
     * 红包列表
     * @return mixed
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\ModelNotFoundException
     * @throws \think\exception\DbException
     */
    public function redPackets()
    {
        $data = $this->request->param();
        $minilist = $this->model->field('mid,name')->where('status', 1)->all();
        $mid = $minilist[0]['mid'];

        if (isset($data['type'])) {
            $map[] = ['red_packets.status', '=', $data['type']];
        }
//        if(isset($data['type'])){
//            $map[] = ['orders.type', '=', $data['type']];
//        }

        if (isset($data['id'])) {
            $mid = $data['id'];
        }
        $map[] = ['mid', '=', $mid];

        $redinfo = Db::view('red_packets', 'id,amount,type,status,create_timestamp')
            ->view('user_info_' . $mid, 'nickname,avator', 'red_packets.uid=user_info_' . $mid . '.uid', 'left')
            ->where($map)
            ->paginate(15, true);

        $this->assign(array(
            'minilist' => $minilist,
            'redlist' => $redinfo,
            'cmini' => $mid,
            'page' => $redinfo->render(),
        ));
        return $this->fetch('red_packets_list');
    }

    /**
     * 拒绝提现
     * @throws \think\Exception\DbException
     */
    public function refuse()
    {
        $order = model('Orders')->get($this->request->param('id'));
        model('RedPackets')->save(array(
            'status' => 0,
        ), array(
            ['uid', '=', $order['uid']],
            ['mid', '=', $order['mid']],
            ['status', '=', 2],
            ['create_timestamp', '<=', $order['create_timestamp']]
        ));
        model('Orders')->save(['status' => 4], ['order_id' => $order['order_id']]);
        $this->success('操作成功');
    }

    /**
     * 确认提现
     * @return \think\response\Json
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\ModelNotFoundException
     * @throws \think\exception\DbException
     */
    public function transfer()
    {
        $order = model('Orders')->get($this->request->param('id'));
        $orderInfo = array(
            'billno' => $order['order_sn'],//商户订单号
            'openid' => $order['pay_sn'],//待提现用户openid
            'money' => intval($order['price'] * 100),//总金额 单位 分
            'desc' => '零钱转账',
        );
        $miniInfo = model('Mini')->where('mid', $order['mid'])->find();
        $wxApp = new \wx\WxApp($miniInfo['app_id'], $miniInfo['mch_id'], $miniInfo['mch_secret']);
        $result = $wxApp->transfers($orderInfo);
        $return_code = trim(strtoupper($result['return_code']));
        $result_code = trim(strtoupper($result['result_code']));
        if ($return_code == 'SUCCESS' && $result_code == 'SUCCESS') {
            model('Orders')->save(['status' => 3], ['order_id' => $order['order_id']]);
            model('RedPackets')->save(array(
                'status' => 0,
            ), array(
                ['uid', '=', $order['uid']],
                ['mid', '=', $order['mid']],
                ['status', '=', 2],
                ['create_timestamp', '<=', $order['create_timestamp']]
            ));
            $this->success('操作成功');
        } else {
            $this->error($result['err_code_des']);
        }
    }

}