<?php

namespace app\api\controller\dev;

use think\Db;
use service\snspay\SnsPayService;
use think\facade\Log;

class Order extends Base
{

    /**
     * 获取订单列表
     * @return \think\response\Json
     */
    public function index()
    {
        Log::record($this->params);
        $this->data['data'] = model('Orders')->getOrderList($this->params);
        return json($this->data);
    }

    /**
     * 生成订单
     * @return \think\response\Json
     * @throws \Exception
     */
    public function order()
    {
      	$config = config('award');
        //根据户口类型获取社保配置
        $conf = getSocialConf($this->params['social_type']);
        if ((isset($this->params['fund_basic']) && $this->params['fund_basic'] < $conf['basic']['fund'][0] && $this->params['fund_basic'] > $conf['basic']['fund'][1])
            || ($this->params['social_basic'] < $conf['basic']['pension'][0] && $this->params['social_basic'] > $conf['basic']['pension'][1])) {
            return json(json_error_exception(20104));
        }
      
        $order_sn = createOrderId();
//        $close_timestamp = date("Y-m-d H:i:s", strtotime("+30 minute"));
        //获取缴费明细
        $caleBill = getCalcBill($this->params);
	
        //生成订单
        $order_data = array(
            'uid' => $this->params['uid'],
            'order_sn' => $order_sn,
            'payment' => $caleBill['total'],
            'service_type' => $caleBill['service_type'],
//            'close_timestamp' => $close_timestamp,
            'status' => 1,
        );
        $order_info = array(
            'uid' => $this->params['uid'],
            'order_sn' => $order_sn,
            'social_basic' => $this->params['social_basic'],
            'charge_details' => json_encode($caleBill),
            'service_charge' => $caleBill['total'] - $caleBill['subtotal'],
            'total_charge' => $caleBill['total'],
        );
        //是否使用直减优惠券
        if($user_discount = Db::name('user_discount')->where(['id'=>$this->params['discount'],'status'=>0])->find()){
            $discount = Db::name('discount')->find($user_discount['discount_id']);
            $order_info['total_charge'] = $order_info['total_charge'] - $discount['money'];
            $order_data['user_discount_id'] = $this->params['discount'];
        }
        //是否使用余额 fee
        // if($this->params['use_balance']){
        //     if($this->params['balance'] == 0 || empty($this->params['balance'])){
        //         return json(json_error_exception('20106'));
        //     }
        // }
      	//手续费
        $order_info['procedure_fee'] =sprintf('%.2f', $order_info['total_charge']/1000);
        $order_info['total_charge'] += $order_info['procedure_fee'];
        if(isset($this->params['fund_basic'])){
            $order_info['fund_basic'] = $this->params['fund_basic'];
        }else{
            $order_info['fund_basic'] = 0;
        }
        if (model('Orders')->createOrder($order_data, $order_info, $caleBill['coupon'], (bool)$caleBill['discounts'])) {
            $this->data['data'] = array(
                'order_sn' => $order_sn,
                'total_charge' => $order_info['total_charge'],
//                'close_timestamp' => $close_timestamp
            );
        } else {
            $this->data = json_error_exception('10101');
        }
        return json($this->data);
    }

    /**
     * 订单详情
     * @return \think\response\Json
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\ModelNotFoundException
     * @throws \think\exception\DbException
     */
    public function detail()
    {
        $orderinfo = model('OrderInfo')->where('order_id', $this->params['id'])->find();
        if (!isset($this->params['id']) || empty($this->params['id']) || $orderinfo->isEmpty()) {
            return json(json_error_exception('20101'));
        }
        $this->data['data'] = array(
            'social_basic' => $orderinfo['social_basic'],
            'fund_basic' => $orderinfo['fund_basic'],
            'charge_details' => json_decode($orderinfo['charge_details'], true),
            'service_charge' => $orderinfo['service_charge'],
            'total_charge' => $orderinfo['total_charge'],
        );
        return json($this->data);
    }

    /**
     * 删除订单-》软删除
     * @return \think\response\Json
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\ModelNotFoundException
     * @throws \think\exception\DbException
     */
    public function delete()
    {
        if (!isset($this->params['order_sn']) || !empty($this->params['order_sn']) || empty($id = model('OrderInfo')->where('order_sn', $this->params['order_sn'])->value('id'))) {
            return json(json_error_exception('20101'));
        }
        if (!model('Orders')->save(['status', 0], ['id', $id])) {
            $this->data = json_error_exception('10101');
        }
        return json($this->data);
    }

    /**
     * 订单支付
     * @return mixed
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\ModelNotFoundException
     * @throws \think\exception\DbException
     */
    public function pay()
    {
        $orderinfo = model('Orders')->where(array(
            ['order_sn','=', $this->params['order_sn']],
            ['uid', '=', $this->params['uid']],
        ))->find();
        $payinfo = model('PayInfo')->where(array(
            ['order_id','=', $orderinfo['id']],
            ['uid', '=', $this->params['uid']],
        ))->find();
        if (!$orderinfo || !$payinfo) {
            return json_error_exception(20102);
        }
        if ($orderinfo['status'] != 1 && $payinfo['pay_status'] != 0) {
            return json_error_exception(20103);
        }
        $name = $this->params['platform'];
        $pay = $this->params['pay'];
        $config = config('platform.' . $name);
        $this->data['data'] = SnsPayService::$name($config)->$pay(array(
            'order_sn' => $orderinfo['order_sn'],
            'money' => $orderinfo['payment'],
            'notify_url' => $_SERVER['REQUEST_SCHEME'].'://'.$_SERVER['HTTP_HOST'].'/dev/notify/'.$name
        ));
        return json($this->data);
    }
}
