<?php

namespace app\api\controller\pc;

use service\SmsService;
use service\sns\SnsService;
use think\facade\Env;
use think\facade\Log;

class Login extends Base
{
    /**
     * @return \think\response\Json|\think\response\Redirect
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\ModelNotFoundException
     * @throws \think\exception\DbException
     */
    public function index()
    {
        echo '你可找不到哦！';
    }
    /**
     * 获取手机验证码
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function getCode()
    {
        header("Access-Control-Allow-Origin: *");
        header('Access-Control-Allow-Headers:*');
        if (isset($this->params['mobile'])) {
            $mobile = $this->params['mobile'];
        }
        if (empty($mobile)) {
            return json(json_error_exception(10101));
        }
        $cache = cache('sms_verify_code_' . $this->request->ip(1));
        if ($cache && time() - $cache['time'] < 60) {
            $this->data = json_error_exception(10102);
        } else {
            $clapi = new SmsService();
            $code = rand_string(4, 1);
            $result = $clapi->sendSMSpc($this->params['mobile'], $code);

            $output = json_decode($result, true);
            if (isset($output['code']) && $output['code'] != '0') {
                $this->data = json_error_exception(20105);
            } else {
                cache('sms_verify_code_' . $this->request->ip(1), array(
                    'code' => $code,
                    'time' => time(),
                ));
            }
        }

        return json($this->data);
    }

    /**
     * 获取js-sdk配置
     * @return \think\response\Json
     */
    public function getJsSdkConf()
    {
        $name = $this->params['platform'];
        $config = config('platform.' . $name);
        $this->data['data'] = SnsService::$name($config)->wxJsSdkConf($this->params['url']);
        return json($this->data);
    }

    public function payInfo()
    {
        if (isset($this->params['order_sn'])) {
            $this->params['money'] = model('Orders')->where('order_sn', $this->params['order_sn'])->value('payment') * 100;
        }
        $name = $this->params['platform'];
        $config = config('platform.' . $name);
        $config['callback'] = url('api/dev.notify/' . $name);
        $res = SnsService::$name($config)->pay($this->params);
        if (is_numeric($res)) {
            $this->data = json_error_exception();
        } else {
            $this->data['data'] = SnsService::$name($config)->pay($this->params);
        }
        return json($this->data);
    }

    public function login($username = '', $code = '', $inviter_code = null)
    {
        header('Access-Control-Allow-Origin:*');
        if (isset($this->params['username'])) {
            $username = $this->params['username'];
        }
        if (isset($this->params['code'])) {
            $code = $this->params['code'];
        }
        if (isset($this->params['inviter_code'])) {
            $inviter_code = $this->params['inviter_code'];
        }
        if (!$username || !$code) {
            return json(json_error_exception('101'));
        }
        //验证码验证
        $cache = cache('sms_verify_code_' . $this->request->ip(1));
        $time = time() - $cache['time'];
        if (empty($cache) || $time > 300 || $cache['code'] != $code) {
            return json(json_error_exception('103'));
        }
        if (\think\facade\Validate::is($username, 'mobile')) {
//            if (cache('mobile_login_code_' . $username) && $password != cache('mobile_login_code_' . $username)) {
            if (cache('mobile_login_code_' . $username)) {
                $this->data = json_error_exception('103');
            }
            $type = 3;
        }
        $user = model('User');
        $data = $user->login(array(
            'username' => $username,
            'password' => $code,
            'inviter_code' => $inviter_code,
        ), $type);
        if ($data['uid'] > 0) {
            $jwt = new \service\EncryptService();
            $return = array(
                'inviter_code' => $data['inviter_code'],
                'uid' => $data['uid'],
                'access_token' => $jwt->createToken($data, 3600, 'access_token'),
                'refresh_token' => $jwt->createToken($data, 86400, 'refresh_token'),
                'expire_in' => 3600,
                're_expire_in' => 86400,
                'token_type' => 'Bearer',
            );
            if (isset($data['social_type']) && $data['social_type']) {
                $return['social_type'] = $data['social_type'];
            }
            $this->data['data'] = $return;
        } else {
            $this->data = json_error_exception('10101');
        }
        return json($this->data);
    }

    public function accessToken()
    {
        $jwt = new \service\EncryptService();
        $checkToken = $jwt->checkToken($this->params['refresh_token']);
        if (is_numeric($checkToken)) {
            $checkToken = $checkToken == 20003 ? 20006 : $checkToken;
            $this->data = json_error_exception($checkToken);
        } else {
            $this->data['data'] = array(
                'access_token' => $jwt->createToken($checkToken['params'], 3600, 'access_token'),
                'expire_in' => 3600,
                'token_type' => 'Bearer',
            );
        }
        return json($this->data);
    }

    /**
     * 查询广告位图
     * @return \think\response\Json
     * @throws \Exception
     */
    public function banner()
    {
        $this->data['data'] = model('Ad')->field('title,cover_url,url,content')->where('status', 1)->order('sort asc')->select()->toArray();
        return json($this->data);
    }

    /**
     * 费用说明
     * @return \think\response\Json
     */
    public function article()
    {
        $this->data['data'] = model('Article')->where('name', $this->params['article_name'])->value('content');
        return json($this->data);
    }

    /**
     * 获取社保缴纳额度
     * @return \think\response\Json
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\ModelNotFoundException
     * @throws \think\exception\DbException
     */
    public function socialLimit()
    {
        $conf = getSocialConf($this->params['social_type']);
        $this->data['data'] = $conf['basic'];
        return json($this->data);
    }


    /**
     * @return \think\response\Json
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\ModelNotFoundException
     * @throws \think\exception\DbException
     */
    public function socialConfig()
    {
        $conf = getSocialConf();
        $min = [];
        foreach ($conf as $v) {
            $total = 0;
            foreach ($v['social'] as $k => $i) {
                if ($k != 'largeMedic') {
                    $total += $v['basic'][$k][0] * ($i[0] + $i[1]);
                } else {
                    $total += $i[0] + $i[1];
                }
            }
            array_push($min, sprintf('%.2f', $total));
        }
        $this->data['data'] = array(
            'service' => $conf['town']['service'],
            'basic' => [
                'social' => $min,
                'fund' => $conf['farmer']['basic']['fund'][0] * ($conf['farmer']['fund']['fund'][0] + $conf['farmer']['fund']['fund'][1])
            ],
            'is_activity' => isActivity()
        );
        return json($this->data);
    }

    /**
     * 代缴明细
     * @return \think\response\Json
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\ModelNotFoundException
     * @throws \think\exception\DbException
     */
    public function calcBill()
    {
        Log::record($this->params['bill_type']);
        $conf = getSocialConf($this->params['social_type']);
        if ((isset($this->params['fund_basic']) && $this->params['fund_basic'] < $conf['basic']['fund'][0] && $this->params['fund_basic'] > $conf['basic']['fund'][1])
            || ($this->params['social_basic'] < $conf['basic']['pension'][0] && $this->params['social_basic'] > $conf['basic']['pension'][1])) {
            return json(json_error_exception(20104));
        }
        $this->data['data'] = getCalcBill($this->params, $this->params['bill_type']);
        return json($this->data);
    }

    public function getSharePic()
    {

        if (!isset($this->params['inviter_code']) || !$this->params['inviter_code']) {
            $this->data = json_error_exception('20201');
        } else {
            $fileName = 'share_' . $this->params['inviter_code'];
            $savePath = Env::get('root_path') . "public/share/$fileName.jpg";
            $retPath = $this->request->domain() . "/share/$fileName.jpg";
            if(file_exists($savePath)){
                $this->data['data'] = $retPath;
            }else {
                require_once Env::get('extend_path') . 'phpqrcode/phpqrcode.php';

                $url = 'http://www.wercf.com/?inviter_code=' . $this->params['inviter_code'];
                $template = Env::get('root_path') . 'public/uploads/share_bg.jpg';
                $font = Env::get('root_path') . 'public/1.ttf';
                $text1 = '50';
                $text2 = $this->params['inviter_code'];
                $temp = 'base.png';
                $qrcode = new \QRcode();
                //生成二维码
                //参数内容:二维码储存内容，生成存储，防错等级，二维码大小，白边大小
                $qrcode->png($url, $temp, QR_ECLEVEL_Q, 8, 1);

                $tpl = imagecreatefromjpeg($template);
                $qr = imagecreatefrompng($temp);
                imagecopyresampled($tpl, $qr, 235, 625, 0, 0, imagesx($qr), imagesx($qr), imagesx($qr), imagesx($qr));

                //奖励金额
                $fontColor = imagecolorallocate($tpl, 255, 255, 255);//字的RGB颜色
                imagettftext($tpl, 30, 0, 390, 470, $fontColor, $font, $text1);

                //邀请码
                $fontColor = imagecolorallocate($tpl, 255, 90, 74);//字的RGB颜色
                imagettftext($tpl, 24, 0, 400, 570, $fontColor, $font, $text2);

                imagepng($tpl, $savePath);
                imagedestroy($tpl);

                $this->data['data'] = $retPath;
            }
        }
        return json($this->data);
    }
}
